@@ -36,7 +36,8 @@ public:
     std::vector<std::map<unsigned, XForm> > &groupXFormsSequence,
     std::vector<std::map<unsigned, unsigned> > &groupMembershipsSequence,
     std::vector<std::list<unsigned> > &documentPageShapeOrders,
-    VSDStyles &styles, VSDStencils &stencils
+    VSDStyles &styles, VSDStencils &stencils, const std::optional<unsigned> &varColInd,
+    const std::optional<unsigned> &varStyInd
   );
 
   void collectDocumentTheme(const VSDXTheme *theme) override;
@@ -45,19 +46,19 @@ public:
   void collectOLEList(unsigned id, unsigned level) override;
   void collectOLEData(unsigned id, unsigned level, const librevenge::RVNGBinaryData &oleData) override;
   void collectEllipse(unsigned id, unsigned level, double cx, double cy, double xleft, double yleft, double xtop, double ytop) override;
-  void collectLine(unsigned level, const boost::optional<double> &strokeWidth, const boost::optional<Colour> &c, const boost::optional<unsigned char> &linePattern,
-                   const boost::optional<unsigned char> &startMarker, const boost::optional<unsigned char> &endMarker,
-                   const boost::optional<unsigned char> &lineCap, const boost::optional<double> &rounding,
-                   const boost::optional<long> &qsLineColour, const boost::optional<long> &qsLineMatrix) override;
-  void collectFillAndShadow(unsigned level, const boost::optional<Colour> &colourFG, const boost::optional<Colour> &colourBG,
-                            const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
-                            const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
-                            const boost::optional<Colour> &shfgc, const boost::optional<double> &shadowOffsetX, const boost::optional<double> &shadowOffsetY,
-                            const boost::optional<long> &qsFc, const boost::optional<long> &qsSc, const boost::optional<long> &qsLm) override;
-  void collectFillAndShadow(unsigned level, const boost::optional<Colour> &colourFG, const boost::optional<Colour> &colourBG,
-                            const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
-                            const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
-                            const boost::optional<Colour> &shfgc) override;
+  void collectLine(unsigned level, const std::optional<double> &strokeWidth, const std::optional<Colour> &c, const std::optional<unsigned char> &linePattern,
+                   const std::optional<unsigned char> &startMarker, const std::optional<unsigned char> &endMarker,
+                   const std::optional<unsigned char> &lineCap, const std::optional<double> &rounding,
+                   const std::optional<long> &qsLineColour, const std::optional<long> &qsLineMatrix) override;
+  void collectFillAndShadow(unsigned level, const std::optional<Colour> &colourFG, const std::optional<Colour> &colourBG,
+                            const std::optional<unsigned char> &fillPattern, const std::optional<double> &fillFGTransparency,
+                            const std::optional<double> &fillBGTransparency, const std::optional<unsigned char> &shadowPattern,
+                            const std::optional<Colour> &shfgc, const std::optional<double> &shadowOffsetX, const std::optional<double> &shadowOffsetY,
+                            const std::optional<long> &qsFc, const std::optional<long> &qsSc, const std::optional<long> &qsLm) override;
+  void collectFillAndShadow(unsigned level, const std::optional<Colour> &colourFG, const std::optional<Colour> &colourBG,
+                            const std::optional<unsigned char> &fillPattern, const std::optional<double> &fillFGTransparency,
+                            const std::optional<double> &fillBGTransparency, const std::optional<unsigned char> &shadowPattern,
+                            const std::optional<Colour> &shfgc) override;
   void collectGeometry(unsigned id, unsigned level, bool noFill, bool noLine, bool noShow) override;
   void collectMoveTo(unsigned id, unsigned level, double x, double y) override;
   void collectLineTo(unsigned id, unsigned level, double x, double y) override;
@@ -77,9 +78,10 @@ public:
   void collectTxtXForm(unsigned level, const XForm &txtxform) override;
   void collectShapesOrder(unsigned id, unsigned level, const std::vector<unsigned> &shapeIds) override;
   void collectForeignDataType(unsigned level, unsigned foreignType, unsigned foreignFormat, double offsetX, double offsetY, double width, double height) override;
-  void collectPageProps(unsigned id, unsigned level, double pageWidth, double pageHeight, double shadowOffsetX, double shadowOffsetY, double scale) override;
+  void collectPageProps(unsigned id, unsigned level, double pageWidth, double pageHeight, double shadowOffsetX, double shadowOffsetY, double scale,
+                        unsigned char drawingScaleUnit, const std::optional<unsigned> variationColorIndex, const std::optional<unsigned> variationStyleIndex) override;
   void collectPage(unsigned id, unsigned level, unsigned backgroundPageID, bool isBackgroundPage, const VSDName &pageName) override;
-  void collectShape(unsigned id, unsigned level, unsigned parent, unsigned masterPage, unsigned masterShape, unsigned lineStyle, unsigned fillStyle, unsigned textStyle) override;
+  void collectShape(unsigned id, unsigned level, unsigned parent, unsigned masterPage, unsigned masterShape, unsigned lineStyle, unsigned fillStyle, unsigned textStyle, const VSDName &aShapeType) override;
   void collectSplineStart(unsigned id, unsigned level, double x, double y, double secondKnot, double firstKnot, double lastKnot, unsigned degree) override;
   void collectSplineKnot(unsigned id, unsigned level, double x, double y, double knot) override;
   void collectSplineEnd() override;
@@ -93,35 +95,35 @@ public:
   void collectUnhandledChunk(unsigned id, unsigned level) override;
 
   void collectText(unsigned level, const librevenge::RVNGBinaryData &textStream, TextFormat format) override;
-  void collectCharIX(unsigned id, unsigned level, unsigned charCount, const boost::optional<VSDName> &font,
-                     const boost::optional<Colour> &fontColour, const boost::optional<double> &fontSize, const boost::optional<bool> &bold,
-                     const boost::optional<bool> &italic, const boost::optional<bool> &underline, const boost::optional<bool> &doubleunderline,
-                     const boost::optional<bool> &strikeout, const boost::optional<bool> &doublestrikeout, const boost::optional<bool> &allcaps,
-                     const boost::optional<bool> &initcaps, const boost::optional<bool> &smallcaps, const boost::optional<bool> &superscript,
-                     const boost::optional<bool> &subscript, const boost::optional<double> &scaleWidth) override;
-  void collectDefaultCharStyle(unsigned charCount, const boost::optional<VSDName> &font, const boost::optional<Colour> &fontColour,
-                               const boost::optional<double> &fontSize, const boost::optional<bool> &bold, const boost::optional<bool> &italic,
-                               const boost::optional<bool> &underline, const boost::optional<bool> &doubleunderline, const boost::optional<bool> &strikeout,
-                               const boost::optional<bool> &doublestrikeout, const boost::optional<bool> &allcaps, const boost::optional<bool> &initcaps,
-                               const boost::optional<bool> &smallcaps, const boost::optional<bool> &superscript, const boost::optional<bool> &subscript,
-                               const boost::optional<double> &scaleWidth) override;
-  void collectParaIX(unsigned id, unsigned level, unsigned charCount, const boost::optional<double> &indFirst,
-                     const boost::optional<double> &indLeft, const boost::optional<double> &indRight, const boost::optional<double> &spLine,
-                     const boost::optional<double> &spBefore, const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
-                     const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr, const boost::optional<VSDName> &bulletFont,
-                     const boost::optional<double> &bulletFontSize, const boost::optional<double> &textPosAfterBullet,
-                     const boost::optional<unsigned> &flags) override;
-  void collectDefaultParaStyle(unsigned charCount, const boost::optional<double> &indFirst, const boost::optional<double> &indLeft,
-                               const boost::optional<double> &indRight, const boost::optional<double> &spLine, const boost::optional<double> &spBefore,
-                               const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
-                               const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr,
-                               const boost::optional<VSDName> &bulletFont, const boost::optional<double> &bulletFontSize,
-                               const boost::optional<double> &textPosAfterBullet, const boost::optional<unsigned> &flags) override;
-  void collectTextBlock(unsigned level, const boost::optional<double> &leftMargin, const boost::optional<double> &rightMargin,
-                        const boost::optional<double> &topMargin, const boost::optional<double> &bottomMargin,
-                        const boost::optional<unsigned char> &verticalAlign, const boost::optional<bool> &isBgFilled,
-                        const boost::optional<Colour> &bgColour, const boost::optional<double> &defaultTabStop,
-                        const boost::optional<unsigned char> &textDirection) override;
+  void collectCharIX(unsigned id, unsigned level, unsigned charCount, const std::optional<VSDName> &font,
+                     const std::optional<Colour> &fontColour, const std::optional<double> &fontSize, const std::optional<bool> &bold,
+                     const std::optional<bool> &italic, const std::optional<bool> &underline, const std::optional<bool> &doubleunderline,
+                     const std::optional<bool> &strikeout, const std::optional<bool> &doublestrikeout, const std::optional<bool> &allcaps,
+                     const std::optional<bool> &initcaps, const std::optional<bool> &smallcaps, const std::optional<bool> &superscript,
+                     const std::optional<bool> &subscript, const std::optional<double> &scaleWidth) override;
+  void collectDefaultCharStyle(unsigned charCount, const std::optional<VSDName> &font, const std::optional<Colour> &fontColour,
+                               const std::optional<double> &fontSize, const std::optional<bool> &bold, const std::optional<bool> &italic,
+                               const std::optional<bool> &underline, const std::optional<bool> &doubleunderline, const std::optional<bool> &strikeout,
+                               const std::optional<bool> &doublestrikeout, const std::optional<bool> &allcaps, const std::optional<bool> &initcaps,
+                               const std::optional<bool> &smallcaps, const std::optional<bool> &superscript, const std::optional<bool> &subscript,
+                               const std::optional<double> &scaleWidth) override;
+  void collectParaIX(unsigned id, unsigned level, unsigned charCount, const std::optional<double> &indFirst,
+                     const std::optional<double> &indLeft, const std::optional<double> &indRight, const std::optional<double> &spLine,
+                     const std::optional<double> &spBefore, const std::optional<double> &spAfter, const std::optional<unsigned char> &align,
+                     const std::optional<unsigned char> &bullet, const std::optional<VSDName> &bulletStr, const std::optional<VSDName> &bulletFont,
+                     const std::optional<double> &bulletFontSize, const std::optional<double> &textPosAfterBullet,
+                     const std::optional<unsigned> &flags) override;
+  void collectDefaultParaStyle(unsigned charCount, const std::optional<double> &indFirst, const std::optional<double> &indLeft,
+                               const std::optional<double> &indRight, const std::optional<double> &spLine, const std::optional<double> &spBefore,
+                               const std::optional<double> &spAfter, const std::optional<unsigned char> &align,
+                               const std::optional<unsigned char> &bullet, const std::optional<VSDName> &bulletStr,
+                               const std::optional<VSDName> &bulletFont, const std::optional<double> &bulletFontSize,
+                               const std::optional<double> &textPosAfterBullet, const std::optional<unsigned> &flags) override;
+  void collectTextBlock(unsigned level, const std::optional<double> &leftMargin, const std::optional<double> &rightMargin,
+                        const std::optional<double> &topMargin, const std::optional<double> &bottomMargin,
+                        const std::optional<unsigned char> &verticalAlign, const std::optional<bool> &isBgFilled,
+                        const std::optional<Colour> &bgColour, const std::optional<double> &defaultTabStop,
+                        const std::optional<unsigned char> &textDirection) override;
   void collectNameList(unsigned id, unsigned level) override;
   void collectName(unsigned id, unsigned level,  const librevenge::RVNGBinaryData &name, TextFormat format) override;
   void collectPageSheet(unsigned id, unsigned level) override;
@@ -132,37 +134,37 @@ public:
 
   // Style collectors
   void collectStyleSheet(unsigned id, unsigned level, unsigned parentLineStyle, unsigned parentFillStyle, unsigned parentTextStyle) override;
-  void collectLineStyle(unsigned level, const boost::optional<double> &strokeWidth, const boost::optional<Colour> &c, const boost::optional<unsigned char> &linePattern,
-                        const boost::optional<unsigned char> &startMarker, const boost::optional<unsigned char> &endMarker,
-                        const boost::optional<unsigned char> &lineCap, const boost::optional<double> &rounding,
-                        const boost::optional<long> &qsLineColour, const boost::optional<long> &qsLineMatrix) override;
-  void collectFillStyle(unsigned level, const boost::optional<Colour> &colourFG, const boost::optional<Colour> &colourBG,
-                        const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
-                        const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
-                        const boost::optional<Colour> &shfgc, const boost::optional<double> &shadowOffsetX, const boost::optional<double> &shadowOffsetY,
-                        const boost::optional<long> &qsFillColour, const boost::optional<long> &qsShadowColour,
-                        const boost::optional<long> &qsFillMatrix) override;
-  void collectFillStyle(unsigned level, const boost::optional<Colour> &colourFG, const boost::optional<Colour> &colourBG,
-                        const boost::optional<unsigned char> &fillPattern, const boost::optional<double> &fillFGTransparency,
-                        const boost::optional<double> &fillBGTransparency, const boost::optional<unsigned char> &shadowPattern,
-                        const boost::optional<Colour> &shfgc) override;
-  void collectCharIXStyle(unsigned id, unsigned level, unsigned charCount, const boost::optional<VSDName> &font,
-                          const boost::optional<Colour> &fontColour, const boost::optional<double> &fontSize, const boost::optional<bool> &bold,
-                          const boost::optional<bool> &italic, const boost::optional<bool> &underline, const boost::optional<bool> &doubleunderline,
-                          const boost::optional<bool> &strikeout, const boost::optional<bool> &doublestrikeout, const boost::optional<bool> &allcaps,
-                          const boost::optional<bool> &initcaps, const boost::optional<bool> &smallcaps, const boost::optional<bool> &superscript,
-                          const boost::optional<bool> &subscript, const boost::optional<double> &scaleWidth) override;
-  void collectParaIXStyle(unsigned id, unsigned level, unsigned charCount, const boost::optional<double> &indFirst,
-                          const boost::optional<double> &indLeft, const boost::optional<double> &indRight, const boost::optional<double> &spLine,
-                          const boost::optional<double> &spBefore, const boost::optional<double> &spAfter, const boost::optional<unsigned char> &align,
-                          const boost::optional<unsigned char> &bullet, const boost::optional<VSDName> &bulletStr,
-                          const boost::optional<VSDName> &bulletFont, const boost::optional<double> &bulletFontSize,
-                          const boost::optional<double> &textPosAfterBullet, const boost::optional<unsigned> &flags) override;
-  void collectTextBlockStyle(unsigned level, const boost::optional<double> &leftMargin, const boost::optional<double> &rightMargin,
-                             const boost::optional<double> &topMargin, const boost::optional<double> &bottomMargin,
-                             const boost::optional<unsigned char> &verticalAlign, const boost::optional<bool> &isBgFilled,
-                             const boost::optional<Colour> &bgColour, const boost::optional<double> &defaultTabStop,
-                             const boost::optional<unsigned char> &textDirection) override;
+  void collectLineStyle(unsigned level, const std::optional<double> &strokeWidth, const std::optional<Colour> &c, const std::optional<unsigned char> &linePattern,
+                        const std::optional<unsigned char> &startMarker, const std::optional<unsigned char> &endMarker,
+                        const std::optional<unsigned char> &lineCap, const std::optional<double> &rounding,
+                        const std::optional<long> &qsLineColour, const std::optional<long> &qsLineMatrix) override;
+  void collectFillStyle(unsigned level, const std::optional<Colour> &colourFG, const std::optional<Colour> &colourBG,
+                        const std::optional<unsigned char> &fillPattern, const std::optional<double> &fillFGTransparency,
+                        const std::optional<double> &fillBGTransparency, const std::optional<unsigned char> &shadowPattern,
+                        const std::optional<Colour> &shfgc, const std::optional<double> &shadowOffsetX, const std::optional<double> &shadowOffsetY,
+                        const std::optional<long> &qsFillColour, const std::optional<long> &qsShadowColour,
+                        const std::optional<long> &qsFillMatrix) override;
+  void collectFillStyle(unsigned level, const std::optional<Colour> &colourFG, const std::optional<Colour> &colourBG,
+                        const std::optional<unsigned char> &fillPattern, const std::optional<double> &fillFGTransparency,
+                        const std::optional<double> &fillBGTransparency, const std::optional<unsigned char> &shadowPattern,
+                        const std::optional<Colour> &shfgc) override;
+  void collectCharIXStyle(unsigned id, unsigned level, unsigned charCount, const std::optional<VSDName> &font,
+                          const std::optional<Colour> &fontColour, const std::optional<double> &fontSize, const std::optional<bool> &bold,
+                          const std::optional<bool> &italic, const std::optional<bool> &underline, const std::optional<bool> &doubleunderline,
+                          const std::optional<bool> &strikeout, const std::optional<bool> &doublestrikeout, const std::optional<bool> &allcaps,
+                          const std::optional<bool> &initcaps, const std::optional<bool> &smallcaps, const std::optional<bool> &superscript,
+                          const std::optional<bool> &subscript, const std::optional<double> &scaleWidth) override;
+  void collectParaIXStyle(unsigned id, unsigned level, unsigned charCount, const std::optional<double> &indFirst,
+                          const std::optional<double> &indLeft, const std::optional<double> &indRight, const std::optional<double> &spLine,
+                          const std::optional<double> &spBefore, const std::optional<double> &spAfter, const std::optional<unsigned char> &align,
+                          const std::optional<unsigned char> &bullet, const std::optional<VSDName> &bulletStr,
+                          const std::optional<VSDName> &bulletFont, const std::optional<double> &bulletFontSize,
+                          const std::optional<double> &textPosAfterBullet, const std::optional<unsigned> &flags) override;
+  void collectTextBlockStyle(unsigned level, const std::optional<double> &leftMargin, const std::optional<double> &rightMargin,
+                             const std::optional<double> &topMargin, const std::optional<double> &bottomMargin,
+                             const std::optional<unsigned char> &verticalAlign, const std::optional<bool> &isBgFilled,
+                             const std::optional<Colour> &bgColour, const std::optional<double> &defaultTabStop,
+                             const std::optional<unsigned char> &textDirection) override;
 
   void collectMetaData(const librevenge::RVNGPropertyList &metaData) override;
 
@@ -197,6 +199,7 @@ private:
   void _flushCurrentPage();
 
   void _handleLevelChange(unsigned level);
+  bool _isDefaultShapeFormat();
 
   void _handleForeignData(const librevenge::RVNGBinaryData &data);
 
@@ -238,6 +241,7 @@ private:
   double m_shadowOffsetX;
   double m_shadowOffsetY;
   double m_scale;
+  unsigned char m_defaultDrawingUnit;
   double m_x;
   double m_y;
   double m_originalX;
@@ -252,6 +256,7 @@ private:
   librevenge::RVNGBinaryData m_currentOLEData;
   librevenge::RVNGPropertyList m_currentForeignProps;
   unsigned m_currentShapeId;
+  unsigned m_parentShapeId;
   unsigned m_foreignType;
   unsigned m_foreignFormat;
   double m_foreignOffsetX;
@@ -295,6 +300,9 @@ private:
   unsigned m_currentStyleSheet;
   VSDStyles m_styles;
 
+  std::optional<unsigned> m_variationColorIndex;
+  std::optional<unsigned> m_variationStyleIndex;
+
   VSDStencils m_stencils;
   const VSDShape *m_stencilShape;
   bool m_isStencilStarted;
@@ -322,6 +330,7 @@ private:
   std::vector<VSDTabSet> m_tabSets;
 
   const VSDXTheme *m_documentTheme;
+  librevenge::RVNGString m_currentShapeType;
 };
 
 } // namespace libvisio
